;; -*- Scheme -*-

;; test for socket-object.scm

;; comlist:remove
(define (remove p l)
  (cond ((null? l) l)
	((eqv? p (car l)) (remove p (cdr l)))
	(else (cons (car l) (remove p (cdr l))))))

;; write out <args>, delimited by spaces, terminated by a newline

(define (blurt . args)
  (letrec ((blurter
	    (lambda (arglist)
	      (if (null? arglist)
		  (newline)
		  (begin (write (car arglist))
			 (display " ")
			 (blurter (cdr arglist)))))))
    (blurter args)))

;; async server.  accepts connections, echos any input from them.
;; shuts down the server when one of the clients sends 'quit<return>'
;; [but won't shut down completely until all clients disconnect]

(define (server-test port)
  (let ((selector (make-selector 256))
	(server (make-socket af_inet sock_stream 0)))
    (let ((channels (list server))
	  (address (string-append "0.0.0.0:" (number->string port))))
      (bind server address)
      (listen server 5)
      (display "listening on ") (display address) (newline)
      ;; select loop
      (let loop ((selected (car (select selector channels '() '() 30.0))))
	(blurt (length selected) "channels selected")
	;; scan the read-fds
	(for-each
	 (lambda (object)
	   (if (eq? object server)
	       ;; a fire on the server object means an incoming client
	       (let ((client (accept server)))
		 (if client
		     (set! channels (cons client channels))))
	       ;; otherwise it's a client channel, read some data
	       ;; from it.
	       (let ((data (recv object 1024)))
		 (cond ((eq? data #f)
			;; eof -> close
			(close object)
			(set! channels (remove object channels)))
		       ((string-ci=? data "quit\r\n")
			;; close this channel _and_ the server
			(close object)
			(set! channels (remove object channels))
			(close server)
			(set! channels (remove server channels)))
		       (else 
			(blurt "fd:" (get-fd object) data))))
	       ))
	 selected
	 )
	(if (> (length channels) 0)
	    (loop (car (select selector channels '() '() 30.0)))
	    ;; return the selector
	    selector)
	)
      )
    )
  )
